% Usage-Options:
% -p: assumed that followed up by a file which contains 'sst(_7_)'.
% -t: TREE PROTOCOLFILE
% -o: TREE_OUTPUT_FILE PROTOCOLFILE		will read the from the protocol spec and write it to  the given output filename
% [no options] PROTOCOLFILE

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% predicates for usability %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
exec_run(ProtocolFile, TreeFile, Verbose) :-
	style_check(-singleton),
	style_check(-discontiguous),
	[main],
	nb_setval(verbose, Verbose),
	read_properties(ProtocolFile, PP),
	see(TreeFile),
	read_term(T, []),
	seen,
	find_strategy_pairs(T, PP, R),
	tell('Result_tree'),
	write_term(R,[]), writeln('.'),
	told,
	halt.
exec_run_without_treefile(ProtocolFile, Verbose) :-
	style_check(-singleton),
	style_check(-discontiguous),
	[main],
	nb_setval(verbose, Verbose),
	read_protocol_file(ProtocolFile, X, PP),
	full_tree(X, T),
	get_variable_tree(T, _, VT),
	tell('Current_tree'),
	write_term(VT,[]), writeln('.'),
	told,
	find_strategy_pairs(VT, PP, R),
	tell('Result_tree'),
	write_term(R,[]), writeln('.'),
 	%write_tree_list(R),
	told,
	halt.
exec_run_without_treefile_outputfile(ProtocolFile, TreeOutputFile, Verbose) :-
	style_check(-singleton),
	style_check(-discontiguous),
	[main],
	nb_setval(verbose, Verbose),
	read_protocol_file(ProtocolFile, X, PP),
	full_tree(X, T),
	get_variable_tree(T, _, VT),
	tell(TreeOutputFile),
	write_term(VT,[]), writeln('.'),
	told,
	find_strategy_pairs(VT, PP, R),
	tell('Result_tree'),
	write_term(R,[]), writeln('.'),
	told,
	halt.

exec_print_tree(T) :-
	style_check(-singleton),
	style_check(-discontiguous),
	[output, mtree],
	see(T),
	read_term(Tree, []),
	seen,
	((is_list(Tree),!,print_tree_list(Tree));
	numNodes(Tree,N),writef("_______Printing tree with %w nodes_________\n",[N]),out(Tree)),
	halt.
exec_print_tree(T, Depth) :-
	style_check(-singleton),
	style_check(-discontiguous),
	[output, mtree],
	see(T),
	read_term(Tree, []),
	seen,
	((is_list(Tree),!, print_tree_list_maxlvl(Tree, Depth));
	numNodes(Tree,N),writef("_______Printing tree with %w nodes_________\n",[N]),out(Tree, Depth)),
	halt.

print_tree_list([]) :- !.
print_tree_list([T|Ts]) :-
	numNodes(T, N),
	writef("_______Printing tree with %w nodes_________\n",[N]),
	out(T),
	print_tree_list(Ts).
print_tree_list_maxlvl([],_) :- !.
print_tree_list_maxlvl([T|Ts],ML) :-
	numNodes(T, N),
	writef("_______Printing tree with %w nodes_________\n",[N]),
	out(T,ML),
	print_tree_list_maxlvl(Ts,ML).

write_tree_list([]) :- !.
write_tree_list([T|Ts]) :-
	write_term(T, []),
	writeln('.'),
	write_tree_list(Ts).






























































