term_atoms(X, 1) :-	var(X), !.
term_atoms(X, 1) :-	atomic(X), !.
term_atoms(X, N) :-
	\+ is_list(X),
	compound(X),
	X =.. [_|XL],
	term_atoms(XL, N), !.
term_atoms([], 0) :- !.
term_atoms([T|Ts], N) :-
	term_atoms(T, TN),
	term_atoms(Ts, TsN),
	N is TN + TsN, !.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%	predicates for common tasks on lists																										%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% count(+List, ) :- count and print occurrences of all members of +List.
count([]).
count(L) :-
	count_wi(L, []).
count_wi([], _).
count_wi([X|Xs], Ig) :-
	not(member(X,Ig)),
	findall(Y, (member(Y,Xs), Y==X), YL),
	length(YL,NumX),
	NumXP is NumX + 1,
	writef("%4r: %w\n",[X,NumXP]),
	count_wi(Xs, [X|Ig]).
count_wi([X|Xs], Ig) :-
	member(X,Ig),
	count_wi(Xs, Ig).

% avg(+List, ?Avg) :- Assuming +List contains only numbers, ?Avg is the average of those.
avg(List, Avg) :-
	sum(List, S),
	length(List, I),
	Avg is S / I.

% sum(+List, ?Sum) :- sum of numbers in +List.
sum([], 0).
sum([X], X) :-
	number(X).
sum([X|Xs], Sum) :-
	sum(Xs, Tmp),
	Sum is X + Tmp.

% take n elements from a set (without putting back)
take_n([],_,[]).
take_n(_, 0, []).
take_n(L, N, [X|R]) :-
	%member(X,L),
	%delete(L,X,LL),
	select(X,L,LL),
	N1 is N - 1,
	take_n(LL,N1,R).

% replace list element
% rle(+Item, +Replacement, +List, ?NewList) :- ...
rle(_,_,[],[]).
rle(A,B,[A|R],[B|Rs]) :-
	rle(A,B,R,Rs).
rle(A,B,[C|R],[C|Rs]) :-
	A \== C,
	rle(A,B,R,Rs).

% return a subset of a list's items
selection([],_).
selection([SL|SLs], L) :-
	element(SL,L,Ls),
	selection(SLs, Ls).
element(X,[X|L],L).
element(X,[_|L],R) :-
	element(X,L,R).

comb(0,_,[]).
comb(K,L,[X|Xs]) :-
	K > 0,
	element(X,L,R),
	K1 is K - 1,
	comb(K1,R,Xs).

% check two lists if there are disjoint
disjoint([], L) :-
	is_list(L).
disjoint(L, []) :-
	is_list(L).
disjoint([M|Ms], [N|Ns]) :-
	not(member(M, [N|Ns])),
	not(member(N, [M|Ms])),
	disjoint(Ms, Ns).

% prefix(+List, +Num, ?PrefixList) :- ?PrefixList is a prefix of +List of length +Num.
prefix([], _, []).
prefix(_, 0, []).
prefix([H|L], N, [H|PL]) :-
	N1 is N - 1,
	prefix(L, N1, PL).

% infix(+List, +IndexList, ?InfixList) :- ?InfixList contains all elements from +List at the indexes given by +IndexList (while ignoring unvalid indexes).
infix(_, [], []).
infix(L, [Index|IndL], R) :- %[X|InfL]) :-
	list_item(L, Index, X),
	not(var(X)),
	infix(L, IndL, InfL),
	append(InfL, [X], R).
infix(L, [Index|IndL], InfL) :-
	list_item(L, Index, X),
	var(X),!,
	infix(L, IndL, InfL).

% tail(+List, -TailElement)
tail([X], X).
tail([_|R], X) :-
	tail(R, X).

% list_item(+List, +Index, ?Item) :- ?Item is the +Index'th element of +List.
list_item([], _, _).
list_item([L|_], 0, L).
list_item([_|Ls], N, X) :-
	N > 0,
	N1 is N - 1,
	list_item(Ls, N1, X).

% replace_in_list(+Element, +Replacement, +List, ?ResultingList) :-
%	either replace +Element in +List by +Replacement,
%	or - if +Replacement is an empty principal - obtain +ResultingList by taking +Element out of +List.
replace_in_list(X, p([]), L, ResL) :-
	append([X], ResL, L), !.
replace_in_list(X, Y, [X|Xs], [Y|Xs]).
replace_in_list(X, Y, [Z|Xs], [Z|Ys]) :-
	replace_in_list(X,Y,Xs,Ys).

% remove_duplicates(+List, ?ResultList) :- ResultList is an ordered set of the items in +List (same order as in +List).
remove_duplicates([], []).
remove_duplicates([M|Ms], Rs) :-
	member(M, Ms),
	remove_duplicates(Ms, Rs), !.
remove_duplicates([M|Ms], [M|R]) :-
	remove_duplicates(Ms, R).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%	predicates for common tasks on our kinds of terms																							%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% reset counters
reset :-
	% verbose-setting set in exec_run-predicates
	nb_setval(gc, 0),		% global counter for renaming variables
	nb_setval(nc, 0),		% node counter
	nb_setval(kc, 0),		% key counter
	nb_setval(sol_count, 0),% solution counter
	nb_setval(pathcount, 0),
	nb_setval(csolve_calls, 0),
	nb_setval(extm,0),
	nb_setval(extd,0),
	nb_setval(extw,0).

% unifier for our own kind of terms
/* what matches:
 * atomic, atomic
 * sc/3, sc/3 (ONLY!)
 * var/1, [anything (but sc/3)]
 * var/2, [anything (but sc/3)]
 * variable, variable
 * composed recursively
 */

/*% atomic
match(T1, T2) :- T1 == T2, !.
% at least one term is a prolog variable
match(T1, T2) :- (var(T1); var(T2)), !.
% our variable terms*/
match(T1, T2) :- \+ \+ T1=T2, !.
% fail case to ensure sc-correctness:
% both terms must be sc to match
match(T1, T2) :-
	T1 =.. [L1|_R1],
	T2 =.. [L2|_R2],
	((L1==sc, L2\==sc); (L1\==sc, L2==sc)),
	!, fail.
match(T1, T2) :- (T1=..[var|_];	T2=..[var|_]), !.
% match correct sc-terms
match(T1, T2) :-
	T1 =.. [sc|R1], length(R1, 3),
	T2 =.. [sc|R2], length(R2, 3),
	match_l(R1, R2), !.
% rest
match(T1, T2) :-
	\+ (atomic(T1); atomic(T2)),
	T1 =.. [H1|R1],
	T2 =.. [H2|R2],
	match(H1, H2),
	match_l(R1, R2).
match_l([], []).
match_l([H1|R1], [H2|R2]) :-
	match(H1,H2),
	match_l(R1, R2).

% find all variables within a given term T
% variables(+ListOfTerms, ?ListOfVariables)
variables(var(X), [X]) :- !.
variables([], []) :- !.
variables([T|Ts], Res) :-
	variables(T, Tmp1),
	variables(Ts, Tmp2),
	append(Tmp1, Tmp2, Res), !.
variables(Term, Res) :-
	Term =.. [_|Ts],
	variables(Ts, Tmp),
	flatten(Tmp, Res), !.

% rename_vars_in_term(+Term, +ListOfVars, ?NewTerm)
rename_vars_in_term(T, [], _RI, T).
rename_vars_in_term(T, [X|Xs], ReplIndex, NT) :-
	rename_single(T,X,ReplIndex,Tmp),
	rename_vars_in_term(Tmp,Xs,ReplIndex,NT).
rename_single(var(X), X, ReplIndex, var(X, ReplIndex)) :- !.
%	nb_getval(gc, GC), !.
rename_single(T, _, _, T) :-
	atomic(T), !.
rename_single(T, X, RI, Res) :-
	T =.. [L|Ls],
	rename_single_l(Ls, X, RI, Ls_),
	Res =.. [L|Ls_].
rename_single_l([], _, _, []).
rename_single_l([T|Ts], X, RI, [Tmp1|Tmp2]) :-
	rename_single(T, X, RI, Tmp1),
	rename_single_l(Ts, X, RI, Tmp2).

